import React, { Component } from 'react';
import { Redirect } from 'react-router-dom';
import _ from 'lodash';
import { publish } from '../../App';
import { InputText } from 'primereact/inputtext';
import { InputNumber } from 'primereact/inputnumber';
import { InputTextarea } from 'primereact/inputtextarea';
import { Checkbox } from 'primereact/checkbox';
import { Dropdown } from 'primereact/dropdown';
import { MultiSelect } from 'primereact/multiselect';
import { Button } from 'primereact/button';
import { Toast } from 'primereact/toast';
import { CustomDialog } from '../../layout/components/CustomDialog';
import { ResourceInputList } from '../../components/Resources/ResourceInputList';
import { appGrowl, setAppGrowl } from '../../layout/components/AppGrowl';
import AppLoader from '../../layout/components/AppLoader';
import PageHeader from '../../layout/components/PageHeader';
import CycleService from '../../services/cycle.service';
import ProjectService from '../../services/project.service';
import UnitConverter from '../../utils/unit.converter';
import UIConstants from '../../utils/ui.constants';
import ReactTooltip from "react-tooltip";
import FormActionbar from '../../components/FormActionbar'; 

/**
 * Component to create a new Project
 */
export class ProjectCreate extends Component {
    constructor(props) {
        super(props);
        this.state = {
            showDialog: false,
            isDirty: false,
            ltaStorage: [],
            isLoading: true,
            dialog: { header: '', detail: ''},
            createAnother: false,
            project: {
                archive_subdirectory: '',
                name: '',
                description: '',
                trigger_priority: 1000,
                rank: 6,
                quota: [],                          // Mandatory Field in the back end, so an empty array is passed
                can_trigger: false,
                auto_pin: false,
                auto_ingest: false,
                piggyback_allowed_tbb: false,
                piggyback_allowed_aartfaac: false                
            },
            projectQuota: {},                       // Resource Allocations
            validFields: {rank:true},               // For Validation
            validForm: false,                       // To enable Save Button
            errors: {},                             // Validation Errors
            periodCategories: [],
            projectCategories: [],
            resources: [],                          // Selected Resources for Allocation
            resourceList: [],                       // Available Resources for Allocation
            cycles: [],
            archive_location:[],
            cluster:[],
            multiCreateCount: 1
        }
        // Validateion Rules
        this.formRules = {
            name: {required: true, message: "Name can not be empty"},
            description: {required: true, message: "Description can not be empty"},
            rank: {required: true, message: "Enter Project Rank. Min-0.00, Max-6.00"},
        };
        this.defaultResourcesEnabled = true;        // This property and functionality to be concluded based on PO input
        this.defaultResources = [{name:'LOFAR Observing Time'}, 
                                    {name:'LOFAR Observing Time prio A'}, 
                                    {name:'LOFAR Observing Time prio B'},
                                    {name:'CEP Processing Time'},
                                    {name:'LTA Storage'},
                                    {name:'Number of triggers'},
                                    {name:'LOFAR Support Time'} ];
        this.projectResourceDefaults = {};          // Default values for default resources
        this.resourceUnitMap = UnitConverter.resourceUnitMap;       // Resource unit conversion factor and constraints
        this.cycleOptionTemplate = this.cycleOptionTemplate.bind(this);         // Template for cycle multiselect
        this.tooltipOptions = UIConstants.tooltipOptions;

        this.setProjectQuotaDefaults = this.setProjectQuotaDefaults.bind(this);
        this.setProjectParams = this.setProjectParams.bind(this);
        this.addNewResource = this.addNewResource.bind(this);
        this.removeResource = this.removeResource.bind(this);
        this.setProjectQuotaParams = this.setProjectQuotaParams.bind(this);
        this.saveProject = this.saveProject.bind(this);
        this.cancelCreate = this.cancelCreate.bind(this);
        this.reset = this.reset.bind(this);
        this.checkIsDirty = this.checkIsDirty.bind(this);
        this.close = this.close.bind(this);
        this.setCreateAnother = this.setCreateAnother.bind(this);
    }

    componentDidMount() {
        ProjectService.getDefaultProjectResources()
            .then(defaults => {
                this.projectResourceDefaults = defaults;
            });
        CycleService.getAllCycles()
            .then(cycles => {
                this.setState({cycles: cycles});
            });
        ProjectService.getProjectCategories()
            .then(categories => {
                this.setState({projectCategories: categories});
            });
        ProjectService.getPeriodCategories()
            .then(categories => {
                this.setState({periodCategories: categories});
            });
        Promise.all([ProjectService.getFileSystem(), ProjectService.getCluster()]).then(response => {
                const options = [];
                response[0].map(fileSystem => {
                    const cluster =  response[1].find(clusterObj => clusterObj.id === fileSystem.cluster_id && clusterObj.archive_site);
                    if (cluster) {
                        fileSystem.label =`${cluster.name} - ${fileSystem.name}`
                        options.push(fileSystem);
                    }
                    return fileSystem;
                });
                this.setState({archive_location: response[0], ltaStorage: options, cluster: response[1] });
            });
        ProjectService.getResources()
            .then(resourceList => {
                const defaultResources = this.defaultResources;
                resourceList = _.sortBy(resourceList, "name");
                const resources = _.remove(resourceList, function(resource) { return _.find(defaultResources, {'name': resource.name})!=null });
                const projectQuota = this.setProjectQuotaDefaults(resources);
                this.setState({resourceList: resourceList, resources: resources, projectQuota: projectQuota, isLoading: false});
            });
    }

    /**
     * Cycle option sub-component with cycle object
     */
    cycleOptionTemplate(option) {
        return (
            <div className="p-clearfix">
                <span style={{fontSize:'1em',float:'right',margin:'1em .5em 0 0'}}>{option.name}</span>
            </div>
        );
    }

    /**
     * Function to set project resource allocation
     * @param {Array} resources 
     */
    setProjectQuotaDefaults(resources) {
        let projectQuota = this.state.projectQuota;
        for (const resource of resources) {
            const conversionFactor = this.resourceUnitMap[resource.quantity_value]?this.resourceUnitMap[resource.quantity_value].conversionFactor:1;
            projectQuota[resource['name']] = this.projectResourceDefaults[resource.name]/conversionFactor;
        }
        return projectQuota;
    }

    /**
     * Function to add new resource to project
     */
    addNewResource(){
        if (this.state.newResource) {
            let resourceList = _.cloneDeep(this.state.resourceList);
            const newResource = _.remove(resourceList, {'name': this.state.newResource});
            let resources = this.state.resources;
            resources.push(newResource[0]);
            if  ( !this.state.isDirty && !_.isEqual(this.state.resourceList, resourceList) ) {
                if(!this.state.isDirty) {
                    publish('edit-dirty', true);
                }
                this.setState({resources: resources, resourceList: resourceList, newResource: null, isDirty: true});
            }   else {
                this.setState({resources: resources, resourceList: resourceList, newResource: null});
            }
        }
    }

        /**
     * Callback function to be called from ResourceInpulList when a resource is removed from it
     * @param {string} name - resource_type_id
     */
    removeResource(name) {
        let resources = this.state.resources;
        let resourceList = this.state.resourceList;
        let projectQuota =  _.cloneDeep(this.state.projectQuota);
        const removedResource = _.remove(resources, (resource) => { return resource.name === name });
        resourceList.push(removedResource[0]);
        resourceList = _.sortBy(resourceList, 'name');
        delete projectQuota[name];
        if  ( !this.state.isDirty && !_.isEqual(this.state.projectQuota, projectQuota) ) {
            if(!this.state.isDirty) {
                publish('edit-dirty', true);
            }
            this.setState({resourceList: resourceList, resources: resources, projectQuota: projectQuota, isDirty: true});
        }   else {
            this.setState({resourceList: resourceList, resources: resources, projectQuota: projectQuota});
        }
        
    }

    /**
     * Function to call on change and blur events from input components
     * @param {string} key 
     * @param {any} value 
     */
    async setProjectParams(key, value, type) {
        let project = this.state.project;
        switch(type) {
            case 'NUMBER': {
                project[key] = value?parseInt(value):0;
                break;
            }
            case 'DECIMAL': {
                const previousValue = this.state.project.rank;
                value = value.replace(/([^0-9.]+)/, "");
                const match = /(\d{0,1})[^.]*((?:\.\d{0,2})?)/g.exec(value);
                const val = match[1] + match[2];
                if (val === '' || (val >= 0 && val <= 6)) {
                    project[key] = val === ''? '' :Number(val);
                }   else {
                    project[key] = previousValue;
                }
                break;
            }
            case 'SUB-DIRECTORY': {
                const directory = value.split(' ').join('_').toLowerCase();
                project[key] = (directory!=="" && !directory.endsWith('/'))? `${directory}/`: `${directory}`;
                break; 
            }
            case 'PROJECT_NAME': {
                let directory = project[key]?project[key].split(' ').join('_').toLowerCase():"";
                if (!project['archive_subdirectory'] || project['archive_subdirectory'] === "" ||
                     project['archive_subdirectory'] === `${directory}/`) {
                    directory = value.split(' ').join('_').toLowerCase();
                    project['archive_subdirectory'] = `${directory}/`;
                }
                project[key] = value;
                break;
            }
            default: {
                project[key] = value;
                break;
            }
        }
        await  this.setState({project: project});
        let validForm = this.validateForm(key);
        if(!this.state.isDirty) {
            publish('edit-dirty', true);
        }
        this.setState({validForm: validForm, isDirty: true});
        
    }

    /**
     * Callback Function to call from ResourceInputList on change and blur events
     * @param {string} key 
     * @param {InputEvent} event 
     */
    setProjectQuotaParams(key, event) {
        let projectQuota = this.state.projectQuota;
        const previousValue = projectQuota[key];
        const eventValue = event.target?.value || event.value
        if (eventValue) {
            let resource = _.find(this.state.resources, {'name': key});
            let newValue = 0;
            if (this.resourceUnitMap[resource.quantity_value] && 
                eventValue.toString().indexOf(this.resourceUnitMap[resource.quantity_value].display)>=0) {
                newValue = _.trim(eventValue.replace(this.resourceUnitMap[resource.quantity_value].display,''));
            }   else {
                newValue = _.trim(eventValue);
            }
            //If value entered is not one decimal number then don't change the value and keep previous value.
            projectQuota[key] = (newValue=="NaN" || !/^[0-9]*(\.[0-9]{0,1})?$/.exec(newValue))?previousValue:(newValue);
        }   else {
            projectQuota[key] = '';
        }
        if  ( !this.state.isDirty && !_.isEqual(previousValue, projectQuota[key]) ) {
            if(!this.state.isDirty) {
                publish('edit-dirty', true);
            }
            this.setState({projectQuota: projectQuota, isDirty: true});
        }   else {
            this.setState({projectQuota: projectQuota});
        }
    }

    /**
     * Validation function to validate the form or field based on the form rules.
     * If no argument passed for fieldName, validates all fields in the form.
     * @param {string} fieldName 
     */
    validateForm(fieldName) {
        let validForm = false;
        let errors = this.state.errors;
        let validFields = this.state.validFields;
        if (fieldName) {
            delete errors[fieldName];
            delete validFields[fieldName];
            if (this.formRules[fieldName]) {
                const rule = this.formRules[fieldName];
                const fieldValue = this.state.project[fieldName];
                if (rule.required) {
                    if (!fieldValue && fieldName !== 'rank') {
                        errors[fieldName] = rule.message?rule.message:`${fieldName} is required`;
                    }   else if (fieldName === 'rank' && fieldValue === '') {
                        errors[fieldName] = rule.message?rule.message:`${fieldName} is required`;
                    }   else {
                        validFields[fieldName] = true;
                    }
                }
            }
        }   else {
            errors = {};
            validFields = {};
            for (const fieldName in this.formRules) {
                const rule = this.formRules[fieldName];
                const fieldValue = this.state.project[fieldName];
                if (rule.required) {
                    if (!fieldValue) {
                        errors[fieldName] = rule.message?rule.message:`${fieldName} is required`;
                    }   else {
                        validFields[fieldName] = true;
                    }
                }
            }
        }
        this.setState({errors: errors, validFields: validFields});
        if (Object.keys(validFields).length === Object.keys(this.formRules).length) {
            validForm = true;
        }
   
        return validForm;
    }
    
    /**
     * Function to call when 'Save' button is clicked to save the project.
     */
    saveProject() {
        if (this.validateForm) {
            let projectQuota = [];
            for (const resource in this.state.projectQuota) {
                let resourceType = _.find(this.state.resources, {'name': resource});
                if(resourceType){
                    let quota = { project: this.state.project.name,
                                    resource_type: resourceType['url'],
                                    value: this.state.projectQuota[resource] * (this.resourceUnitMap[resourceType.quantity_value]?this.resourceUnitMap[resourceType.quantity_value].conversionFactor:1)};
                    projectQuota.push(quota);
                }
            }
            let project = { ...this.state.project };
            project['rank'] = project.rank.toFixed(2);
            ProjectService.saveProject(project, this.defaultResourcesEnabled?projectQuota:[])
                .then(project => {
                    if (project.url) {
                       if (appGrowl === null) {
                            setAppGrowl(this.growl)
                        }
                        if (project.isQuotaCreated) {
                            if (this.defaultResourcesEnabled) {
                                appGrowl.show({severity: 'success', summary: 'Success', detail: 'Project saved successfully.'});
                            }   else {
                                appGrowl.show({severity: 'success', summary: 'Success', detail: 'Project saved successfully with default Resource allocations.'});
                            }
                        }   else {
                            appGrowl.show({severity: 'info', summary: 'Success', detail: 'Project saved successfully, but resource allocation not saved.'});
                        }
                        if (this.state.createAnother) {
                            this.setState({project:project, isDirty: false},() => publish('edit-dirty', false));
                            this.reset();
                        }   else {
                            this.setState({project:project, isDirty: false, redirect: `/project`},() => publish('edit-dirty', false));
                        }
                        
                    }   else {
                        appGrowl.show({severity: 'error', summary: 'Error Occured', detail: 'Unable to save Project'});
                        this.setState({errors: project, isDirty: false});
                        publish('edit-dirty', false);
                    }
                });
        }
    }

     /**
     * warn before cancel the page if any changes detected 
     */
    checkIsDirty() {
        if( this.state.isDirty ){
            this.setState({showDialog: true});
        } else {
            this.cancelCreate();
        }
    }
    
    close() {
        this.setState({showDialog: false});
    }

    /**
     * Function to cancel form creation and navigate to other page/component
     */
    cancelCreate() {
        publish('edit-dirty', false);
        if (this.state.multiCreateCount > 1) {
            this.props.history.go(this.state.multiCreateCount*-1);
        }   else {
            this.props.history.length>1?this.state.isDirty?this.props.history.go(-2):this.props.history.go(-1):this.props.history.push(`/project`);
        }
        this.setState({showDialog: false});
    }

    /**
     * Reset function to be called to reset the form fields
     */
    reset() {
        if (this.defaultResourcesEnabled) {
            let prevResources = this.state.resources;
            let multiCreateCount = this.state.multiCreateCount;
            let resourceList = [];
            let resources = [];
            if (resources) {
                const defaultResources = this.defaultResources;
                resourceList = _.sortBy(prevResources.concat(this.state.resourceList), "name");
                resources = _.remove(resourceList, function(resource) { return _.find(defaultResources, {'name': resource.name})!=null });
            }
            const projectQuota = this.setProjectQuotaDefaults(resources);
            this.setState({
                dialog: { header: '', detail: ''},
                project: {
                    url: '',
                    name: '',
                    description: '',
                    trigger_priority: 1000,
                    rank: 0,
                    quota: [],
                    archive_location: null,
                    archive_subdirectory:"",
                    auto_ingest: false,
                    piggyback_allowed_tbb: false,
                    piggyback_allowed_aartfaac: false
                },
                projectQuota: projectQuota,
                validFields: {},
                validForm: false,
                errors: {},
                dialogVisible: false,
                resources: resources,
                resourceList: resourceList,
                multiCreateCount: ++multiCreateCount
            });
        }   else {
            this.setState({redirect: `/project/edit/${this.state.project.name}`})
        }
    }

    /**
     * Select/Unselect the create another check box
     * @param {*} e 
     */
    setCreateAnother(e) {
        this.setState({'createAnother': e.target.checked})
    }

    render() {
        if (this.state.redirect) {
            return <Redirect to={ {pathname: this.state.redirect} }></Redirect>
        }
        return (
            <React.Fragment>
                <Toast ref={(el) => this.growl = el} />
                 <PageHeader className="defaultpageHeader" location={this.props.location} title={'Project - Add'} actions={[{icon:'pi pi-times', title:'Click to Close Project', 
                   type: 'buttonv2',  actOn: 'click', props:{ callback: this.checkIsDirty }}]}/>
                { this.state.isLoading ? <AppLoader /> :
                <>
                <div style={{marginBottom: '2em'}}>
                    <div className="p-fluid">
                        <div className="p-field p-grid" style={{display: 'none'}}>
                            <label htmlFor="projectId" className="col-lg-2 col-md-2 col-sm-12">URL </label>
                            <div className="col-lg-3 col-md-3 col-sm-12">
                                <input id="projectId" data-testid="projectId" value={this.state.project.url} />
                            </div>
                        </div>
                        <div className="p-field p-grid">
                            <label htmlFor="projectName" className="col-lg-2 col-md-2 col-sm-12">Name <span style={{color:'red'}}>*</span></label>
                            <div className="col-lg-3 col-md-3 col-sm-12">
                                <InputText className={this.state.errors.name ?'input-error':''} id="projectName" data-testid="name" 
                                            tooltip="Enter name of the project" tooltipOptions={this.tooltipOptions} maxLength="128"
                                            value={this.state.project.name} 
                                            onChange={(e) => this.setProjectParams('name', e.target.value,'PROJECT_NAME')}
                                            onBlur={(e) => this.setProjectParams('name', e.target.value,'PROJECT_NAME')}/>
                                <label className={this.state.errors.name?"error":"info"}>
                                    {this.state.errors.name ? this.state.errors.name : "Max 128 characters"}
                                </label>
                            </div>
                            <div className="col-lg-1 col-md-1 col-sm-12"></div>
                            <label htmlFor="description" className="col-lg-2 col-md-2 col-sm-12">Description <span style={{color:'red'}}>*</span></label>
                            <div className="col-lg-3 col-md-3 col-sm-12">
                                <InputTextarea className={this.state.errors.description ?'input-error':''} rows={3} cols={30} 
                                            tooltip="Short description of the project" tooltipOptions={this.tooltipOptions} maxLength="128"
                                            data-testid="description" value={this.state.project.description} 
                                            onChange={(e) => this.setProjectParams('description', e.target.value)}
                                            onBlur={(e) => this.setProjectParams('description', e.target.value)}/>
                                <label className={this.state.errors.description ?"error":"info"}>
                                    {this.state.errors.description ? this.state.errors.description : "Max 255 characters"}
                                </label>
                            </div>
                        </div>
                        <div className="p-field p-grid">
                            <label htmlFor="triggerPriority" className="col-lg-2 col-md-2 col-sm-12">Trigger Priority </label>
                            <div className="col-lg-3 col-md-3 col-sm-12" data-testid="trig_prio">
                                <InputNumber inputId="trig_prio" name="trig_prio" value={this.state.project.trigger_priority} 
                                        tooltip="Priority of this project with respect to triggers" tooltipOptions={this.tooltipOptions}
                                        mode="decimal" showButtons min={0} max={10000} step={10} useGrouping={false}
                                        onChange={(e) => this.setProjectParams('trigger_priority', e.value)}
                                        onBlur={(e) => this.setProjectParams('trigger_priority', e.target.value, 'NUMBER')} />
                                
                                <label className="error">
                                    {this.state.errors.trigger_priority ? this.state.errors.trigger_priority : ""}
                                </label>
                            </div>
                            <div className="col-lg-1 col-md-1 col-sm-12"></div>
                            <label htmlFor="trigger" className="col-lg-2 col-md-2 col-sm-12">Allows Trigger Submission</label>
                            <div className="col-lg-3 col-md-3 col-sm-12" data-testid="trigger">
                                <Checkbox inputId="trigger" role="trigger" 
                                        tooltip="Is this project allowed to supply observation requests on the fly, possibly interrupting currently running observations (responsive telescope)?" 
                                        tooltipOptions={this.tooltipOptions}
                                        checked={this.state.project.can_trigger} onChange={e => this.setProjectParams('can_trigger', e.target.checked)}></Checkbox>
                            </div>
                        </div>
                        <div className="p-field p-grid">
                            <label htmlFor="projCat" className="col-lg-2 col-md-2 col-sm-12">Project Category </label>
                            <div className="col-lg-3 col-md-3 col-sm-12" data-testid="projCat" >
                                <Dropdown inputId="projCat" optionLabel="value" optionValue="url"  
                                        tooltip="Project Category" tooltipOptions={this.tooltipOptions}
                                        value={this.state.project.project_category} 
                                        options={this.state.projectCategories} 
                                        onChange={(e) => {this.setProjectParams('project_category', e.value)}} 
                                        placeholder="Select Project Category" />
                            </div>
                            <div className="col-lg-1 col-md-1 col-sm-12"></div>
                            <label htmlFor="periodCategory" className="col-lg-2 col-md-2 col-sm-12">Period Category</label>
                            <div className="col-lg-3 col-md-3 col-sm-12">
                                <Dropdown data-testid="period-cat" id="period-cat" optionLabel="value" optionValue="url" 
                                        tooltip="Period Category" tooltipOptions={this.tooltipOptions}
                                        value={this.state.project.period_category} 
                                        options={this.state.periodCategories} 
                                        onChange={(e) => {this.setProjectParams('period_category',e.value)}} 
                                        placeholder="Select Period Category" />
                            </div>
                        </div>
                        <div className="p-field p-grid">
                            <label htmlFor="triggerPriority" className="col-lg-2 col-md-2 col-sm-12">Cycle(s)</label>
                            <div className="col-lg-3 col-md-3 col-sm-12">
                                <MultiSelect data-testid="cycle" id="cycle" optionLabel="name" optionValue="url" filter={true}
                                        tooltip="Cycle(s) to which this project belongs" tooltipOptions={this.tooltipOptions}
                                        value={this.state.project.cycles} 
                                        options={this.state.cycles} 
                                        onChange={(e) => {this.setProjectParams('cycles',e.value)}} 
                                 />
                            </div>
                            <div className="col-lg-1 col-md-1 col-sm-12"></div>
                            <label htmlFor="projRank" className="col-lg-2 col-md-2 col-sm-12">Project Rank <span style={{color:'red'}}>*</span></label>
                            <div className="col-lg-3 col-md-3 col-sm-12" data-testid="proj-rank" >
                                <input type="number"
                                    data-for="reacttooltip"
                                    data-iscapture="true"
                                    data-tip="Priority of this project w.r.t. other projects. Projects can interrupt observations of lower-priority projects. Min-0.00, Max-6.00"
                                    name="rank" data-testid="rank" id="proj-rank"
                                    className="p-inputtext p-component"
                                    value={this.state.project.rank}
                                    step="0.01"
                                    onChange={(e)=> this.setProjectParams('rank', e.target.value, "DECIMAL")}/>
                                <label className="error">
                                    {this.state.errors.rank ? this.state.errors.rank : ""}
                                </label>
                            </div>
                        </div>
                        <div className="p-field p-grid">
                            <label htmlFor="ltaStorage" className="col-lg-2 col-md-2 col-sm-12">LTA Storage Location</label>
                            <div className="col-lg-3 col-md-3 col-sm-12" >
                                <Dropdown inputId="ltaStore"
                                        optionValue="url"
                                        tooltip="LTA Storage" tooltipOptions={this.tooltipOptions}
                                        value={this.state.project.archive_location}
                                        options={this.state.ltaStorage}
                                        onChange={(e) => this.setProjectParams('archive_location', e.target.value)}
                                        placeholder="Select LTA Storage" />
                            </div>
                            <div className="col-lg-1 col-md-1 col-sm-12"></div>
                            <label htmlFor="preventdeletionafteringest" className="col-lg-2 col-md-2 col-sm-12">Prevent Automatic Deletion After Ingest</label>
                            <div className="col-lg-3 col-md-3 col-sm-12" data-testid="preventdeletionafteringest">
                                <Checkbox inputId="preventdeletionafteringest" role="preventdeletionafteringest" 
                                        tooltip="Prevent automatic deletion after ingest" 
                                        tooltipOptions={this.tooltipOptions}
                                        checked={this.state.project.auto_pin} onChange={e => this.setProjectParams('auto_pin', e.target.checked)}></Checkbox>
                            </div>
                        </div>
                        <div className="p-field p-grid">
                            <label htmlFor="ingestautomatically" className="col-lg-2 col-md-2 col-sm-12">Ingest Automatically</label>
                            <div className="col-lg-3 col-md-3 col-sm-12" data-testid="ingestautomatically">
                                <Checkbox inputId="ingestautomatically" role="ingestautomatically" 
                                        tooltip="Ingest Automatically" 
                                        tooltipOptions={this.tooltipOptions}
                                        checked={this.state.project.auto_ingest} onChange={e => this.setProjectParams('auto_ingest', e.target.checked)}></Checkbox>
                            </div>
                            <div className="col-lg-1 col-md-1 col-sm-12"></div>
                            <label htmlFor="allowcommensaltbb" className="col-lg-2 col-md-2 col-sm-12">Allow commensal TBB</label>
                            <div className="col-lg-3 col-md-3 col-sm-12" data-testid="allowcommensaltbb">
                                <Checkbox inputId="allowcommensaltbb" role="allowcommensaltbb" 
                                        tooltip="Allow commensal TBB" 
                                        tooltipOptions={this.tooltipOptions}
                                        checked={this.state.project.piggyback_allowed_tbb} onChange={e => this.setProjectParams('piggyback_allowed_tbb', e.target.checked)}></Checkbox>
                            </div>
                        </div>
                        <div className="p-field p-grid">
                            <label htmlFor="allowcommensalaartfaac" className="col-lg-2 col-md-2 col-sm-12">Allow commensal AARTFAAC</label>
                            <div className="col-lg-3 col-md-3 col-sm-12" data-testid="allowcommensalaartfaac">
                                <Checkbox inputId="allowcommensalaartfaac" role="allowcommensalaartfaac" 
                                        tooltip="Allow commensal AARTFAAC" 
                                        tooltipOptions={this.tooltipOptions}
                                        checked={this.state.project.piggyback_allowed_aartfaac} onChange={e => this.setProjectParams('piggyback_allowed_aartfaac', e.target.checked)}></Checkbox>
                            </div>
                        </div>
                        {this.defaultResourcesEnabled && this.state.resourceList &&
                            <div className="p-fluid">
                                <div className="p-field p-grid">
                                    <div className="col-lg-2 col-md-2 col-sm-112">
                                        <h5 data-testid="resource_alloc">Resource Allocations</h5>
                                    </div>
                                    <div className="col-lg-3 col-md-3 col-sm-10">
                                        <Dropdown optionLabel="name" optionValue="name" 
                                            tooltip="Resources to be allotted for the project" 
                                            tooltipOptions={this.tooltipOptions}
                                            value={this.state.newResource} 
                                            options={this.state.resourceList} 
                                            onChange={(e) => {this.setState({'newResource': e.value})}}
                                            placeholder="Add Resources" />
                                    </div>
                                    <div className="col-lg-2 col-md-2 col-sm-2">
                                    <Button label="" className="p-button-primary" icon="pi pi-plus" onClick={this.addNewResource} disabled={!this.state.newResource} data-testid="add_res_btn" />
                                    </div>
                                </div>
                                <div className="p-field p-grid resource-input-grid">
                                    <ResourceInputList list={this.state.resources} unitMap={this.resourceUnitMap} 
                                    quota={this.state.projectQuota} callback={this.setProjectQuotaParams} 
                                    removeInputCallback={this.removeResource} />
                                </div>
                            </div>                            
                        }
                        <ReactTooltip id="reacttooltip" place={'left'}  type={'dark'} effect={'solid'} multiline={true} />
                    </div>
                </div>
                <div className="p-breadcrumb footer-long-div actions">
                    <FormActionbar createAnotherCallBack={this.setCreateAnother} createAnother={this.state.createAnother} 
                    tooltip="Select checkbox to create another Project after saving this project"
                    onSubmit={this.saveProject} disableSaveBtn={!this.state.validForm} onCancel={this.checkIsDirty} />
                </div>
                </>
                }

                {/* Dialog component to show messages and get input */}
                <div className="p-grid" data-testid="confirm_dialog">
                    <CustomDialog type="confirmation" visible={this.state.showDialog} width="40vw" data-testid='dialog'
                        header={'Add Project'} message={'Do you want to discard your changes? A new project will not be created.'} 
                        content={''} onClose={this.close}
                        actions={ [ {id:"yes", title: 'Discard', callback: this.cancelCreate, className:'act-btn-dispose'},
                            {id:"no", title: 'Cancel', className:'act-btn-cancel', callback: this.close} ]}  >
                    </CustomDialog>
                </div>
                
            </React.Fragment>
        );
    }
}
