import React, {Component} from 'react';
import {Link, Redirect} from 'react-router-dom'
import moment from 'moment';
import _ from 'lodash';

import { Chips } from 'primereact/chips';
import { TieredMenu } from 'primereact/tieredmenu';

import ResourceDisplayList from '../../components/Resources/ResourceDisplayList';
import AppLoader from '../../layout/components/AppLoader';
import PageHeader from '../../layout/components/PageHeader';
import ProjectService from '../../services/project.service';
import UnitConverter from '../../utils/unit.converter';
import SchedulingUnitList from './../Scheduling/SchedulingUnitList';
import UIConstants from '../../utils/ui.constants';
import AuthUtil from '../../utils/auth.util';
import AccessDenied from '../../layout/components/AccessDenied';
import { DataTable } from 'primereact/datatable';
import { Column } from 'primereact/column';
import { appGrowl } from '../../layout/components/AppGrowl';
import { Button } from 'primereact/button';
import { Dialog } from 'primereact/dialog';
import { RadioButton } from 'primereact/radiobutton';
import { CustomDialog } from '../../layout/components/CustomDialog';
import { Checkbox } from 'primereact/checkbox';

/**
 * Component to view the details of a project
 */
export class ProjectView extends Component {
    constructor(props) {
        super(props);
        this.state = {
            ltaStorage: [],
            isLoading: true,
            project:'',
            showStatusUpdateDialog: false
        };
        if (this.props.match.params.id) {
            this.state.projectId  = this.props.match.params.id;
        }   else if (this.props.location.state && this.props.location.state.id) {
            this.state.projectId = this.props.location.state.id;
        }
        this.cancelView = this.cancelView.bind(this);
        this.showStatusChangeDialog = this.showStatusChangeDialog.bind(this);
        this.confirmStatusChange  = this.confirmStatusChange.bind(this);
        this.closeDialog = this.closeDialog.bind(this);
        this.submitStatusChange = this.submitStatusChange.bind(this);
        this.state.redirect = this.state.projectId?"":'/project'         // If no project id is passed, redirect to Project list page
        this.resourceUnitMap = UnitConverter.resourceUnitMap;       // Resource unit conversion factor and constraints
    }

    async componentDidMount() {
        const projectId = this.state.projectId;
        if (projectId) {
            AuthUtil.getUserPermissionByModuleId('project', projectId)
            .then (permission => {
                this.setState({ permissionById: permission});
                if (permission[projectId] && permission[projectId].view) {
                    this.getProjectDetails(projectId);
                    Promise.all([ProjectService.getFileSystem(), ProjectService.getCluster()]).then(response => {
                        const options = {};
                        if (response.length === 2 && response[0] && response[1]) {
                            response[0].map(fileSystem => {
                                const cluster =  response[1].filter(clusterObj => clusterObj.id === fileSystem.cluster_id && clusterObj.archive_site);
                                if (cluster.length) {
                                    options[fileSystem.url] = `${cluster[0].name} - ${fileSystem.name}`
                                }
                                return fileSystem;
                            });
                            this.setState({archive_location: response[0], ltaStorage: options, cluster: response[1] });
                        }
                    });
                }
            });
            this.statusOptions = await ProjectService.getProjectStates();
        }   else {
            this.setState({redirect: "/not-found"});
        }
    }

    cancelView(){
        this.props.history.length>1?this.props.history.goBack():this.props.history.push(`/project`);
    }
    /**
     * To get the project details from the backend using the service
     * 
     */
    async getProjectDetails() {
        let project = await ProjectService.getProjectDetails(this.state.projectId);
        let projectQuota = [];
        let resources = []; 
        if (project) {
            // If resources are allocated for the project quota fetch the resources master from the API
            if (project.quota) {
                resources = await ProjectService.getResources();
            }
            // For every project quota, get the resource type & assign to the resource variable of the quota object
            for (const id of project.quota_ids) {
                let quota = await ProjectService.getProjectQuota(id);
                for(const archiveId of quota.project_quota_archive_location_ids) {
                    const projectArchive = await ProjectService.getProjectQuotaArchiveLocation(archiveId);
                    if(projectArchive) {
                        project.archive_location = projectArchive.archive_location;
                    }
                }
                let resource = _.find(resources, ['name', quota.resource_type_id]);
                quota.resource = resource?resource:{name:quota.resource_type_id};
                projectQuota.push(quota);
            }
            this.setState({project: project, projectQuota: projectQuota, isLoading: false});
        }   else {
            this.setState({redirect: "../../not-found"})
        }
    }

    /**
     * Content for confirmation dialog
     * @returns Content to be dispalyed in the confirmation dialog
     */
    getSUDialogContent = () => {
        let changedData = [];
        changedData.push({
            name: this.state.project.name,
            project_state_value: this.state.changedStatus.value
        });
        
        return (
                <div style={{ marginTop: '1em' }}>
                    <b>Do you want to change the status of this project?</b>
                    <div style={{ marginTop: '1em' }}>                 
                        <DataTable value={changedData} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                            <Column field="name" header="Project Name"></Column>
                            <Column field="project_state_value" header="Status"></Column>
                        </DataTable>                  
                    </div>
                </div>
        )
    }

    /**
     * 
     * @returns Content to be displayed in dialog after updating the staus in backend
     */
    getSUResponseDialogContent = () => {
        let changedData = [];
        for (const row of this.state.statusChangeResponse) {
            changedData.push(row);
        }
        return (
                <div style={{ marginTop: '1em' }}>
                    <b></b>
                    <div style={{ marginTop: '1em' }}>                 
                        <DataTable value={this.state.statusChangeResponse} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                            <Column field="name" header="Project Name"></Column>
                            <Column field="status" header="Project Status"></Column>
                            <Column field="message" header="Update Status"></Column>
                        </DataTable>                  
                    </div>
                </div>
        )
    }

    /**
     * Show dialog with all available status options
     */
    showStatusChangeDialog() {
        this.setState({ showStatusUpdateDialog: !this.state.showStatusUpdateDialog, changedStatus: null });
    }

    /**
     * Show confirmation dialog for status change
     */
    confirmStatusChange () {
        let dialog = {};
        dialog.type = "confirmation";
        dialog.header = `Confirm Project Status change`;
        dialog.content = this.getSUDialogContent;
        dialog.actions = [{
            id: 'yes', title: 'Yes', callback: () => {
                this.submitStatusChange();
            }
        },
        { id: 'no', title: 'No',className:'act-btn-cancel' , callback: this.closeDialog }];
        dialog.onSubmit = () => {}
        dialog.width = '25vw';
        dialog.showIcon = false;
        dialog.dialogVisible = true;
        this.setState({dialog: dialog})          
    }

    closeDialog () {
        let dialog = this.state.dialog;
        dialog.dialogVisible = false
        this.setState({dialog: dialog, showStatusChangeDialog: false});
        return false;
    }

    /**
     * Updating status in backend on confirmation and displaying response of the updation
     */
    async submitStatusChange () {
        let statusChangeResponse = []
        let project = this.state.project;
        project.project_state = this.state.changedStatus.url
        project.project_state_value = this.state.changedStatus.value
        let response = await ProjectService.updateProject(project.name, project)
        if(response.isUpdated) {
            statusChangeResponse.push({
                name: response.name,
                status: response.project_state_value,
                message: 'Success'
            })
        }
        else {
            statusChangeResponse.push({
                name: response.name,
                status: response.project_state_value,
                message: 'Failed'
            })
            appGrowl.show({ severity: 'error', summary: 'Failed to update project status'});
        }
        this.setState({statusChangeResponse: statusChangeResponse})
        let dialog = this.state.dialog;
        dialog.type = "confirmation";
        dialog.header = `Project Status Update Info`;
        dialog.content = this.getSUResponseDialogContent;
        dialog.actions = [{
            id: 'yes', title: 'Close',className:'act-btn-cancel', callback: () => {
                this.closeDialog();
                this.setState({showStatusUpdateDialog: false});
            }
        }];
        dialog.onSubmit = () => {
            this.closeDialog();
            this.setState({showStatusUpdateDialog: false});
        }
        dialog.width = '30vw';
        dialog.showIcon = false;
        dialog.dialogVisible = true; 
        this.setState({dialog: dialog, changedStatus: {}})
    }



    render() {
        const footer = (
            <div>
                <Button label="Save" className="p-button-primary p-mr-2" icon="pi pi-check" disabled={!this.state.changedStatus} onClick={this.confirmStatusChange } data-testid="save-btn" />
                <Button label="Cancel" className="act-btn-cancel mr-0" icon="pi pi-times" onClick={() => this.setState({showStatusUpdateDialog: false, changedStatus: {}})} />
            </div>
        );
        if (this.state.redirect) {
            return <Redirect to={ {pathname: this.state.redirect} }></Redirect>
        }
        return (
            <React.Fragment>
                {this.state.permissionById &&
                    <>
                        {this.state.permissionById[this.state.projectId] && this.state.permissionById[this.state.projectId].view ?
                        <>
                        <TieredMenu className="app-header-menu" model={this.menuOptions} popup ref={el => this.optionsMenu = el} />
                        <PageHeader location={this.props.location} title={'Project - Details'} 
                                    actions={[{icon: 'fa fa-tag', 
                                                title: this.state.permissionById[this.state.projectId].edit?'Update Project Status':"Don't have permission to update status",
                                                disabled: this.state.permissionById[this.state.projectId].edit?!this.state.permissionById[this.state.projectId].edit:true,
                                                type: 'button', actOn: 'click', props: { callback: this.showStatusChangeDialog }},
                                                {icon: 'fa-edit',
                                                title: this.state.permissionById[this.state.projectId].edit?'Click to Edit Project':"Don't have permission to edit",
                                                type:'link',
                                                disabled: this.state.permissionById[this.state.projectId].edit?!this.state.permissionById[this.state.projectId].edit:true, 
                                                props : { pathname: `/project/edit/${this.state.project.name}`, 
                                                        state: {id: this.state.project?this.state.project.name:''&& this.state.project}}},
                                                {icon:'fa-window-close',title: 'Click to Close Project View', type: 'button',  actOn: 'click', props:{ callback: this.cancelView }}]}/>
                        { this.state.isLoading && <AppLoader /> }
                        { this.state.project &&
                                <div className="main-content">
                                    <div className="p-grid">
                                        <label className="col-lg-2 col-md-2 col-sm-12">Name</label>
                                        <span className="col-lg-4 col-md-4 col-sm-12">{this.state.project.name}</span>
                                        <label className="col-lg-2 col-md-2 col-sm-12">Description</label>
                                        <span className="col-lg-4 col-md-4 col-sm-12">{this.state.project.description}</span>
                                    </div>
                                    <div className="p-grid">
                                        <label className="col-lg-2 col-md-2 col-sm-12">Created At</label>
                                        <span className="col-lg-4 col-md-4 col-sm-12">{moment.utc(this.state.project.created_at).format(UIConstants.CALENDAR_DATETIME_FORMAT)}</span>
                                        <label className="col-lg-2 col-md-2 col-sm-12">Updated At</label>
                                        <span className="col-lg-4 col-md-4 col-sm-12">{moment.utc(this.state.project.updated_at).format(UIConstants.CALENDAR_DATETIME_FORMAT)}</span>
                                    </div>
                                    <div className="p-grid">
                                        <label className="col-lg-2 col-md-2 col-sm-12">Trigger Priority</label>
                                        <span className="col-lg-4 col-md-4 col-sm-12">{this.state.project.trigger_priority}</span>
                                        <label className="col-lg-2 col-md-2 col-sm-12">Allows Trigger Submission</label>
                                        <Checkbox disabled className="p-checkbox-disabled col-lg-4 col-md-4 col-sm-12" checked={this.state.project.can_trigger?true: false}></Checkbox>
                                    </div>
                                    <div className="p-grid">
                                        <label className="col-lg-2 col-md-2 col-sm-12">Project Category</label>
                                        <span className="col-lg-4 col-md-4 col-sm-12">{this.state.project.project_category_value}</span>
                                        <label className="col-lg-2 col-md-2 col-sm-12">Period Category</label>
                                        <span className="col-lg-4 col-md-4 col-sm-12">{this.state.project.period_category_value}</span>
                                    </div>
                                    <div className="p-grid">
                                        <label className="col-lg-2 col-md-2 col-sm-12">Cycles</label>
                                        <Chips className="col-lg-4 col-md-4 col-sm-12 chips-readonly" disabled value={this.state.project.cycles_ids}></Chips>
                                        <label className="col-lg-2 col-md-2 col-sm-12">Project Rank</label>
                                        <span className="col-lg-4 col-md-4 col-sm-12">{this.state.project.rank}</span>
                                    </div>
                                    <div className="p-grid">
                                        <label className="col-lg-2 col-md-2 col-sm-12">Project Status</label>
                                        <div className="col-lg-4 col-md-4 col-sm-12" style={{marginBottom:"12px"}}>
                                            <span className={"tag-content p-chips-token-label proj-"+this.state.project.project_state_value}>{this.state.project.project_state_value}</span>
                                        </div>
                                        <label className="col-lg-2 col-md-2 col-sm-12">LTA Storage Location</label>
                                        <span className="col-lg-4 col-md-4 col-sm-12">{this.state.ltaStorage[this.state.project.archive_location]}</span>
                                    </div>
                                    <div className="p-grid">
                                        <label className="col-lg-2 col-md-2 col-sm-12">Prevent Automatic Deletion After Ingest</label>
                                        <Checkbox disabled className="p-checkbox-disabled col-lg-4 col-md-4 col-sm-12" checked={this.state.project.auto_pin?true: false}></Checkbox>
                                        <label className="col-lg-2 col-md-2 col-sm-12">Ingest Automatically</label>
                                        <Checkbox disabled className="p-checkbox-disabled col-lg-4 col-md-4 col-sm-12" checked={this.state.project.auto_ingest?true: false}></Checkbox>
                                    </div>
                                    <div className="p-grid">
                                        <label className="col-lg-2 col-md-2 col-sm-12">Allow commensal TBB</label>
                                        <Checkbox disabled className="p-checkbox-disabled col-lg-4 col-md-4 col-sm-12" checked={this.state.project.piggyback_allowed_tbb?true: false}></Checkbox>
                                        <label className="col-lg-2 col-md-2 col-sm-12">Allow commensal AARTFAAC</label>
                                        <Checkbox disabled className="p-checkbox-disabled col-lg-4 col-md-4 col-sm-12" checked={this.state.project.piggyback_allowed_aartfaac?true: false}></Checkbox>

                                    </div>
                                    <div className="p-fluid">
                                        <div className="p-field p-grid">
                                            <div className="col-lg-3 col-md-3 col-sm-12">
                                                <h5 data-testid="resource_alloc">Resource Allocations</h5>
                                            </div>
                                        </div>
                                    </div>
                                    {this.state.projectQuota.length===0 && 
                                        <div className="p-field p-grid">
                                            <div className="col-lg-12 col-md-12 col-sm-12">
                                                <span>Reosurces not yet allocated. 
                                                    <Link to={{ pathname: `/project/edit/${this.state.project.name}`, state: {id: this.state.project?this.state.project.name:''}}} title="Edit Project" > Click</Link> to add.
                                                </span>
                                            </div>
                                        </div>
                                    }
                                    <div className="p-field p-grid resource-input-grid">
                                        <ResourceDisplayList quota={this.state.projectQuota}  unitMap={this.resourceUnitMap} />
                                    </div>
                                    {/* Show Schedule Unit belongs to Project */}
                                    <div className="p-fluid">
                                        <div className="p-field p-grid">
                                            <div className="col-lg-3 col-md-3 col-sm-12">
                                                <h5 data-testid="resource_alloc">Scheduling Unit - List</h5>
                                            </div>
                                        </div>
                                    </div>
                                    <div className="view-list">
                                        <SchedulingUnitList project={this.state.project.name} hideProjectColumn 
                                            allowRowSelection={true} ref={suList => {this.suList = suList}} />
                                    </div>
                                </div>
                        }
                        {this.state.showStatusUpdateDialog &&
                            <div>
                                <Dialog header={`Update Project Status`}
                                    footer={footer} maximizable={false}
                                    visible={this.state.showStatusUpdateDialog}  maximized={false} position="center" style={{ width: '20vw' }}
                                    onHide={() => { this.setState({ showStatusUpdateDialog: false, changedStatus: {} }) }}
                                    className="content_dlg">
                                        <div style={{ width: '100%' }}>
                                            <div className="p-fluid">
                                                <div className="p-grid p-field" style={{ paddingLeft: '15px' }}>Select the status and click &apos;Save&apos; to update.</div>
                                                {this.statusOptions.map((option) => {
                                                    return (
                                                        <div className="p-col-12" style={{width: "100%"}} key={option.value}>
                                                            <RadioButton
                                                                inputId={option.value}
                                                                name={option.value}
                                                                value={option.value}
                                                                onChange={() => this.setState({ changedStatus: option })}
                                                                checked={this.state.changedStatus && this.state.changedStatus.value === option.value}
                                                            />
                                                            <label htmlFor={option.value} className='p-radiobutton-label'>
                                                                {_.startCase(option.value)}
                                                            </label>
                                                        </div>
                                                    )
                                                })}
                                            </div>
                                        </div>
                                </Dialog>
                                <CustomDialog type="confirmation" visible={this.state.dialog?.dialogVisible}
                                    header={this.state.dialog?.header} message={this.state.dialog?.detail} actions={this.state.dialog?.actions}
                                    content={this.state.dialog?.content} width={this.state.dialog?.width} showIcon={this.state.dialog?.showIcon}
                                    onClose={this.closeDialog} onCancel={this.closeDialog} onSubmit={this.submitStatusChange} />
                            </div>
                        }
                    </>: <AccessDenied/>}
                </>}
            </React.Fragment>
        );
    }
}