import $RefParser from 'json-schema-ref-parser';
import _ from 'lodash';
import Auth from '../authenticate/auth';

import axios from "axios"
const refParser = new $RefParser();
const fs = require('fs');
const path = require('path');

let COMMON_SCHEMA_TEMPLATE = '';
let isCommonSchemaApiCallInProgress = false;
let apiCommonSchemaCallPromise = null;
/**
 * Utility Service to fetch miscellaneous data from the server
 */
const UtilService = {
  /** Function to fetch current UTC time from the server which is the System Clock */
  getUTC: async function () {
    try {
      const url = `/api/util/utc`;
      const response = await axios.get(url);
      return response.data;
    } catch (error) {
      console.error(error);
    }
  },

    /** Function to fetch the equivalent LST (Local Sidereal Time) for a UTC timestamp. 
   * 'timestamp' should be a string in "YYYY-MM-DDTHH:mm:ss" format. 
   * Data fetched is stored in local storage to avoid repeated request to server for the same value.*/
  getLST: async (timestamp, stations = ["CS002"]) => {
    try {
      const stationstring = stations.join(',')
      const  storageKey =   'UTC_LST_MAP_V3_' + stationstring;
      // give cached answer
      let localUtcLstMap = localStorage.getItem(storageKey); // open local storage.  Note, if a station get's moved to a new location... this will give wrong answer :)
      if (localUtcLstMap) {                                  // if the local storage exist..
        localUtcLstMap = JSON.parse(localUtcLstMap);         // read in the entire storage
        if (localUtcLstMap[timestamp]) {                     // see if this timestamp already has an answer
          return Promise.resolve(localUtcLstMap[timestamp]); // if so, return this answer
        }
      }

      localUtcLstMap = localUtcLstMap || {};
      const url = `/api/util/lst?timestamp=${timestamp}&stations=${stationstring}`;
      const response = await axios.get(url);
      /* The response will be in 
      ..
           "UTC": "2023-11-13T10:52:19Z",
            "LST": ..
                "CS002": "14:48:59"
        ..
      ..
      format
      */
      localUtcLstMap[timestamp] = response;
      localStorage.setItem('UTC_LST_MAP_V3' + stationstring, JSON.stringify(localUtcLstMap));
      return response;
    } catch (error) {
      return null;
    }
  },
  /** Function to fetch sun timings from the backend for single station. */
  getSunTimings: async (timestamp, stations) => {
    try {
      stations = stations || "CS002";
      let stationTimestamp = (stations ? `${stations}-` : "") + timestamp;
      let localSunTimeMap = localStorage.getItem('SUN_TIME_MAP_V3');
      if (localSunTimeMap) {
        localSunTimeMap = JSON.parse(localSunTimeMap);
        if (localSunTimeMap[stationTimestamp]) {
          return Promise.resolve(localSunTimeMap[stationTimestamp]);
        }
      } else {
        localSunTimeMap = {};
      }
      const url = `/api/util/sun_rise_and_set?stations=${stations}&timestamps=${timestamp}`;
      const stationSunTimings = (await axios.get(url)).data;
      localSunTimeMap[stationTimestamp] = stationSunTimings;
      localStorage.setItem('SUN_TIME_MAP_V3', JSON.stringify(localSunTimeMap));
      return stationSunTimings;
    } catch (error) {
      console.error(error);
      return null;
    }
  },
  /** Gets all reservations in the system */
  getReservations: async () => {
    try {
      const reservations = (await axios.get("/api/reservation")).data.results;
      return reservations;
    } catch (error) {
      console.error(error);
    }
  },
/** Gets all reservations in the system */
getStations: async () => {
  try {
    const stations = (await axios.get("/api/util/stations")).data;
    return stations;
  } catch (error) {
    console.error(error);
  }
},
  /** Gets reservation templates in the system */
  getReservationTemplates: async () => {
    try {
      const templates = (await axios.get("/api/reservation_template")).data.results;
      return templates;
    } catch (error) {
      console.error(error);
    }
  },
  resolveSchema: async function (schema) {
    let properties = schema.properties;
    schema.definitions = schema.definitions ? schema.definitions : {};
    if (properties) {
      for (const propertyKey in properties) {
        let property = properties[propertyKey];
        if (property["$ref"] && !property["$ref"].startsWith("#")) {    // 1st level reference of the object
          const refUrl = property["$ref"];
          let newRef = refUrl.substring(refUrl.indexOf("#"));
          let defKey = refUrl.substring(refUrl.lastIndexOf("/") + 1);
          schema.definitions[defKey] = (await refParser.resolve(refUrl, { cache: { http: 5, https: 5 } })).get(refUrl);
          property["$ref"] = newRef;
          if (schema.definitions[defKey].type && (schema.definitions[defKey].type === 'array'
            || schema.definitions[defKey].type === 'object')) {
            let resolvedItems = await this.resolveSchema(schema.definitions[defKey]);
            if (resolvedItems.items && resolvedItems.items['$ref'] && _.keys(resolvedItems.definitions).length === 1) {
              const resolvedRefKey = resolvedItems.items['$ref'];
              resolvedItems.items = resolvedItems.definitions[resolvedRefKey.substring(resolvedRefKey.lastIndexOf("/") + 1)];
            } else {
              schema.definitions = { ...schema.definitions, ...resolvedItems.definitions };
            }
            delete resolvedItems['definitions'];
          }
        } else if (property["type"] === "array") {             // reference in array items definition
          let resolvedItems = await this.resolveSchema(property["items"]);
          schema.definitions = { ...schema.definitions, ...resolvedItems.definitions };
          delete resolvedItems['definitions'];
          property["items"] = resolvedItems;
        } else if (property["type"] === "object" && property.properties) {
          property = await this.resolveSchema(property);
          schema.definitions = { ...schema.definitions, ...property.definitions };
          delete property['definitions'];
        }
        properties[propertyKey] = property;
      }
    } else if (schema["oneOf"] || schema["anyOf"]) {             // Reference in OneOf/anyOf array
      let defKey = schema["oneOf"] ? "oneOf" : "anyOf";
      let resolvedOneOfList = []
      for (const oneOfProperty of schema[defKey]) {
        const resolvedOneOf = await this.resolveSchema(oneOfProperty);
        resolvedOneOfList.push(resolvedOneOf);
        if (resolvedOneOf.definitions) {
          schema.definitions = { ...schema.definitions, ...resolvedOneOf.definitions };
        }
      }
      schema[defKey] = resolvedOneOfList;
    } else if (schema["$ref"] && !schema["$ref"].startsWith("#")) {   //reference in oneOf list item
      const refUrl = schema["$ref"];
      let newRef = refUrl.substring(refUrl.indexOf("#"));
      let defKey = refUrl.substring(refUrl.lastIndexOf("/") + 1);
      schema.definitions[defKey] = (await refParser.resolve(refUrl, { cache: { http: 5, https: 5 } })).get(refUrl);
      if (schema.definitions[defKey].properties || schema.definitions[defKey].type === "object"
        || schema.definitions[defKey].type === "array") {
        let property = await this.resolveSchema(schema.definitions[defKey]);
        schema.definitions = { ...schema.definitions, ...property.definitions };
        delete property['definitions'];
        schema.definitions[defKey] = property;
      }
      schema["$ref"] = newRef;
    } else if (schema["type"] === "array") {             // reference in array items definition
      let resolvedItems = await this.resolveSchema(schema["items"]);
      schema.definitions = { ...schema.definitions, ...resolvedItems.definitions };
      delete resolvedItems['definitions'];
      schema["items"] = resolvedItems;
    }
    return schema;
  },
  findObject: function (arrObj, obj, aoProp, oProp) {
    return arrObj.find((ao) => ao[aoProp] === obj[oProp]);
  },
  findObjectIndex: function (arrObj, obj, aoProp, oProp) {
    return arrObj.findIndex((ao) => ao[aoProp] === obj[oProp]);
  },
  filterByObject: function (arrObj, action) {
    return arrObj.filter((ao) => ao.action === action);
  },
  /**
   * Function calls the API to resolve all references fo the template schema and returns the schema JSON object
   * @param {string} templateUrl - url of the template
   * @returns - resolved schema JSON object.
   */
  getResolvedSchema: async (templateUrl) => {
    let schema = null;
    try {
      schema = (await axios.get(`${templateUrl}/ref_resolved_schema`)).data;
    } catch (error) {
      console.error(error);
    }
    return schema;
  },
  /**
   * Function to store the UI attributes in browser local storage. 'data' to be passed as JSON object 
   * with key as the name, value object to be stored or fetched, type - method get/set/remove
   * @param {Object} data - JSON object 
   * @returns JSON object
   */
  localStore: function (data) {
    // Last viewed options of users are stored as preference for the users when they open the screen next time
    // Values are stored for each user separately.
    let user = Auth.getUser();
    user = user ? user.name : "";
    let { type, key, value } = data;
    key = `${user}${user ? "-" : ""}${key}`;
    if (type === 'set') {
      try {
        localStorage.setItem(key, JSON.stringify(value));
      }
      catch (e)
      {
      console.log(e,data);
      }
      
    } else if (type === 'get') {
      const localStorageItem = localStorage.getItem(key);
      if (localStorageItem && localStorageItem !== "undefined") {
        return JSON.parse(localStorageItem);
      } else {
        return {};
      }
    } else if (type === 'remove') {
      localStorage.removeItem(key);
    }
  },
  getPriorityQueueType: async function () {
    try {
      const response = await axios.get('/api/priority_queue_type/');
      return response.data.results;
    } catch (error) {
      console.error('[UtilService.getPriorityQueueType]', error);
    }
  },
  getProjectRoles: async function () {
    try {
      const response = await axios.get('/api/project_role/');
      return response.data.results;
    } catch (error) {
      console.error('[UtilService.getProjectRoles]', error);
      return [];
    }
  },
  /**
   * Get Dynamic Schediling On/Off Status
   * @returns JSON object
   */
  getDynamicSchedulerStatus: async function () {
    try {
      const url = `/api/setting/dynamic_scheduling_enabled`;
      const response = await axios.get(url);
      return response.data;
    } catch (error) {
      console.error('[getDynamicSchedulerStatus]', "Mistake", error);
      return null;
    }
  },
  /**
   * Get FixedTime Scheduling On/Off Status
   * @returns JSON object
   */
  getFixedTimeSchedulerStatus: async function () {
    try {
      const url = `/api/setting/fixed_time_scheduling_enabled`;
      const response = await axios.get(url);
      return response.data;
    } catch (error) {
      console.error('[getFixedTimeSchedulerStatus]', "Mistake", error);
      return null;
    }
  },
  getSubSystemScheduler: async function () {
    try {
      const url = '/api/subsystem/scheduler';
      const response = await axios.get(url);
      return response.data;
    } catch (error) {
      console.error('[getSubStystemScheduler]', error);
      return null;
    }
  },
  /**
   * Update Switch On/Off status of Dynamic Scheduling
   * @param {Object} JSON object 
   * @returns 
   */
  updateDynamicSchedulerStatus: async function (response) {
    try {
      const url = `/api/setting/dynamic_scheduling_enabled`;
      response = await axios.patch(url, response);
      return response.data;
    } catch (error) {
      console.error('[updateDynamicSchedulerStatus]', "Mistake", error);
      return null
    }
  },
  /**
   * Update Switch On/Off status of FixedTime Scheduling
   * @param {Object} JSON object
   * @returns
   */
  updateFixedTimeSchedulerStatus: async function (response) {
    try {
      const url = `/api/setting/fixed_time_scheduling_enabled`;
      response = await axios.patch(url, response);
      return response.data;
    } catch (error) {
      console.error('[updateFixedTimeSchedulerStatus]', "Mistake", error);
      return null
    }
  },
  getAllFilesList: async (dir) => {
    let files = await fs.promises.readdir(dir);
    files = await Promise.all(files.map(async file => {
      const filePath = path.join(dir, file);
      const stats = await fs.promises.stat(filePath);
      if (stats.isDirectory()) return UtilService.getAllFilesList(filePath);
      else if (stats.isFile()) return filePath;
    }));

    return files.reduce((all, folderContents) => all.concat(folderContents), []);
  },

  
  getCommonSchemaTemplate: async function () {
      if (!isCommonSchemaApiCallInProgress && (COMMON_SCHEMA_TEMPLATE === undefined || COMMON_SCHEMA_TEMPLATE === '')) {
        isCommonSchemaApiCallInProgress = true; // Acquire the lock
  
        const url = '/api/common_schema_template?name=stations&ordering=-id&state=active';
        apiCommonSchemaCallPromise = axios.get(url)
          .then(response => {
            if (response.data.results.length > 0) {
              COMMON_SCHEMA_TEMPLATE = response.data.results[0];
            }
          })
          .catch(error => {
            console.error('[getCommonSchemaTemplate]', 'Mistake', error);
          })
          .finally(() => {
            isCommonSchemaApiCallInProgress = false; // Release the lock
          });
      }
    // Wait for the promise to resolve, even if the API call is not made
    await apiCommonSchemaCallPromise;
    return COMMON_SCHEMA_TEMPLATE;
  }
}

export default UtilService;